function [ue,un,uv,DV]=CDdisp(X,Y,Z,X0,Y0,depth,dV,nu)
% CDdisp
% calculates the displacements associated with a center of dilatation in a 
% half-space.
% 
% CD: Center of Dilatation
% EFCS: Earth-Fixed Coordinate System
%
% INPUTS
% X, Y and Z:
% Coordinates of calculation points in EFCS (East, North, Up). X, Y and Z
% must have the same size.
%
% X0, Y0 and depth:
% Horizontal coordinates (in EFCS) and depth of the CD. The depth must be a
% positive value. X0, Y0 and depth have the same unit as X, Y and Z.
%
% dV:
% Volume change of the CD. The volume change has the unit of the 
% calculation point coordinates to the power of 3. 
% Note that dV is different from the source potency (DV) in the outputs!
%
% nu:
% Poisson's ratio.
% 
% 
% OUTPUTS
% ue, un and uv:
% Calculated displacement vector components in EFCS. ue, un and uv have the
% same unit as X, Y and Z in inputs.
% 
% DV:
% Potency of the CD. The potency has the unit of volume (the unit of the
% calculation point coordinates to the power of 3).
% Note that DV is different from the volume change, dV in the inputs!
%
%
% Example: Calculate and plot the vertical displacements on a regular grid
%
% [X,Y,Z] = meshgrid(-7:.02:7,-5:.02:5,-1);
% X0 = 0.5; Y0 = -0.25; depth = 2.75; dV = 0.001; nu = 0.25; 
% [ue,un,uv,DV] = CDdisp(X,Y,Z,X0,Y0,depth,dV,nu);
% figure
% surf(X,Y,reshape(uv,size(X)),'edgecolor','none')
% view(2)
% axis equal
% axis tight
% set(gcf,'renderer','painters')

% Reference journal article:
% Okada, Y., 1992. Internal deformation due to shear and tensile faults in 
% a half-space, Bull. seism. Soc. Am., 82(2), 10181040.

% Copyright (c) 2016 Mehdi Nikkhoo
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files
% (the "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to permit
% persons to whom the Software is furnished to do so, subject to the
% following conditions:
%
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
% NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
% OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
% USE OR OTHER DEALINGS IN THE SOFTWARE.

% I appreciate any comments or bug reports.

% Mehdi Nikkhoo
% Created: 2016.10.19
% Last modified: 2016.10.19
%
% Section 2.1, Physics of Earthquakes and Volcanoes
% Department 2, Geophysics
% Helmholtz Centre Potsdam
% German Research Centre for Geosciences (GFZ)
%
% email:
% mehdi.nikkhoo@gfz-potsdam.de
% mehdi.nikkhoo@gmail.com
%
% website:
% http://www.volcanodeformation.com

% calculate bulk modulus (K) to shear modulus (mu) ratio
Km = 2*(1+nu)/3/(1-2*nu);
% calculate the CD potency
DV = (1+4/3/Km)*dV; % This holds ONLY for spherical sources!

% Note: For a center of dilatation M0 = DV*K!
sf = Km/2/pi;
DVs = DV*sf;

x = X(:)-X0;
y = Y(:)-Y0;
z = Z(:);

dp = depth-z;
dn = depth+z;
rp = (x.^2+y.^2+dp.^2).^0.5;
rn = (x.^2+y.^2+dn.^2).^0.5;

alpha = 1/2/(1-nu);

[ue_Ap,un_Ap,uv_Ap] = uA(x,y,dp,rp,alpha);
[ue_An,un_An,uv_An] = uA(x,y,dn,rn,alpha);
[ue_B,un_B,uv_B] = uB(x,y,dp,rp,alpha);
[ue_C,un_C,uv_C] = uC(x,y,dp,rp,alpha);

ue = DVs*(ue_Ap-ue_An+ue_B+z.*ue_C);
un = DVs*(un_Ap-un_An+un_B+z.*un_C);
uv = DVs*(uv_Ap-uv_An+uv_B+z.*uv_C);


function [ue,un,uv]=uA(x,y,d,r,alpha)
ue = -(1-alpha)/2*x./r.^3;
un = -(1-alpha)/2*y./r.^3;
uv = -(1-alpha)/2*d./r.^3;

function [ue,un,uv]=uB(x,y,d,r,alpha)
ue = (1-alpha)/alpha*x./r.^3;
un = (1-alpha)/alpha*y./r.^3;
uv = (1-alpha)/alpha*d./r.^3;

function [ue,un,uv]=uC(x,y,d,r,alpha)
C3 = 1-3*d.^2./r.^2;
ue = (1-alpha)*3*x.*d./r.^5;
un = (1-alpha)*3*y.*d./r.^5;
uv = (1-alpha)*C3./r.^3;
