function [Stress,Strain,DV]=CDstress(X,Y,Z,X0,Y0,depth,dV,mu,lambda)
% CDstress
% calculates the strains and stresses associated with a center of 
% dilatation in a half-space.
% 
% CD: Center of Dilatation
% EFCS: Earth-Fixed Coordinate System
%
% INPUTS
% X, Y and Z:
% Coordinates of calculation points in EFCS (East, North, Up). X, Y and Z
% must have the same size.
%
% X0, Y0 and depth:
% Horizontal coordinates (in EFCS) and depth of the CD. The depth must be a
% positive value. X0, Y0 and depth have the same unit as X, Y and Z.
%
% dV:
% Volume change of the CD. The volume change has the unit of the 
% calculation point coordinates to the power of 3. 
% Note that dV is different from the source potency (DV) in the outputs!
%
% mu and lambda:
% Lame constants.
% 
% 
% OUTPUTS
% Stress:
% Calculated stress tensor components in EFCS. The six columns of Stress
% are Sxx, Syy, Szz, Sxy, Sxz and Syz, respectively. The stress components
% have the same unit as Lame constants.
%
% Strain:
% Calculated strain tensor components in EFCS. The six columns of Strain
% are Exx, Eyy, Ezz, Exy, Exz and Eyz, respectively. The strain components
% are dimensionless.
% 
% DV:
% Potency of the CD. The potency has the unit of volume (the unit of the
% calculation point coordinates to the power of 3).
% Note that DV is different from the volume change, dV in the inputs!
%
%
% Example: Calculate and plot the first component of stress tensor on a 
% regular grid
%
% [X,Y,Z] = meshgrid(-7:.02:7,-5:.02:5,-1);
% X0 = 0.5; Y0 = -0.25; depth = 2.75; DV = 0.001; mu = 0.3e11; 
% lambda = 0.3e11;
% [Stress,Strain,DV] = CDstress(X,Y,Z,X0,Y0,depth,dV,mu,lambda);
% figure
% surf(X,Y,reshape(Stress(:,1),size(X)),'edgecolor','none')
% view(2)
% axis equal
% axis tight
% set(gcf,'renderer','painters')

% Reference journal article:
% Okada, Y., 1992. Internal deformation due to shear and tensile faults in 
% a half-space, Bull. seism. Soc. Am., 82(2), 10181040.

% Copyright (c) 2016 Mehdi Nikkhoo
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files
% (the "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to permit
% persons to whom the Software is furnished to do so, subject to the
% following conditions:
%
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
% NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
% OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
% USE OR OTHER DEALINGS IN THE SOFTWARE.

% I appreciate any comments or bug reports.

% Mehdi Nikkhoo
% Created: 2016.10.19
% Last modified: 2016.10.19
%
% Section 2.1, Physics of Earthquakes and Volcanoes
% Department 2, Geophysics
% Helmholtz Centre Potsdam
% German Research Centre for Geosciences (GFZ)
%
% email:
% mehdi.nikkhoo@gfz-potsdam.de
% mehdi.nikkhoo@gmail.com
%
% website:
% http://www.volcanodeformation.com

% calculate bulk modulus
K = lambda+2*mu/3;

% calculate the CD potency
DV = (1+4*mu/3/K)*dV; % This holds ONLY for spherical sources!

% Note: For a center of dilatation M0 = DV*K!
sf = K/mu/2/pi;
DVs = DV*sf;

x = X(:)-X0;
y = Y(:)-Y0;
z = Z(:);

dp = depth-z;
dn = depth+z;
rp = (x.^2+y.^2+dp.^2).^0.5;
rn = (x.^2+y.^2+dn.^2).^0.5;

alpha = (lambda+mu)/(lambda+2*mu);

% ru_rx
[rue_rxAp,run_rxAp,ruv_rxAp] = ruA_rx(x,y,dp,rp,alpha);
[rue_rxAn,run_rxAn,ruv_rxAn] = ruA_rx(x,y,dn,rn,alpha);
[rue_rxB,run_rxB,ruv_rxB] = ruB_rx(x,y,dp,rp,alpha);
[rue_rxC,run_rxC,ruv_rxC] = ruC_rx(x,y,dp,rp,alpha);

rue_rx = DVs*(rue_rxAp-rue_rxAn+rue_rxB+z.*rue_rxC);
run_rx = DVs*(run_rxAp-run_rxAn+run_rxB+z.*run_rxC);
ruv_rx = DVs*(ruv_rxAp-ruv_rxAn+ruv_rxB+z.*ruv_rxC);

% ru_ry
[rue_ryAp,run_ryAp,ruv_ryAp] = ruA_ry(x,y,dp,rp,alpha);
[rue_ryAn,run_ryAn,ruv_ryAn] = ruA_ry(x,y,dn,rn,alpha);
[rue_ryB,run_ryB,ruv_ryB] = ruB_ry(x,y,dp,rp,alpha);
[rue_ryC,run_ryC,ruv_ryC] = ruC_ry(x,y,dp,rp,alpha);

rue_ry = DVs*(rue_ryAp-rue_ryAn+rue_ryB+z.*rue_ryC);
run_ry = DVs*(run_ryAp-run_ryAn+run_ryB+z.*run_ryC);
ruv_ry = DVs*(ruv_ryAp-ruv_ryAn+ruv_ryB+z.*ruv_ryC);

% ru_rz
[rue_rzAp,run_rzAp,ruv_rzAp] = ruA_rz(x,y,dp,rp,alpha);
[rue_rzAn,run_rzAn,ruv_rzAn] = ruA_rz(x,y,dn,rn,alpha);
[rue_rzB,run_rzB,ruv_rzB] = ruB_rz(x,y,dp,rp,alpha);
[rue_rzC,run_rzC,ruv_rzC] = ruC_rz(x,y,dp,rp,alpha);
[ue_C,un_C,uv_C] = uC(x,y,dp,rp,alpha);

rue_rz = DVs*(rue_rzAp+rue_rzAn+rue_rzB+ue_C+z.*rue_rzC);
run_rz = DVs*(run_rzAp+run_rzAn+run_rzB+un_C+z.*run_rzC);
ruv_rz = DVs*(ruv_rzAp+ruv_rzAn+ruv_rzB+uv_C+z.*ruv_rzC);

% Calculate the strain tensor components
Exx = rue_rx;
Eyy = run_ry;
Ezz = ruv_rz;
Exy = (rue_ry+run_rx)/2;
Exz = (rue_rz+ruv_rx)/2;
Eyz = (run_rz+ruv_ry)/2;

% Calculate the stress tensor components
Sxx = 2*mu*Exx+lambda*(Exx+Eyy+Ezz);
Syy = 2*mu*Eyy+lambda*(Exx+Eyy+Ezz);
Szz = 2*mu*Ezz+lambda*(Exx+Eyy+Ezz);
Sxy = 2*mu*Exy;
Sxz = 2*mu*Exz;
Syz = 2*mu*Eyz;

Strain = [Exx,Eyy,Ezz,Exy,Exz,Eyz];
Stress = [Sxx,Syy,Szz,Sxy,Sxz,Syz];


function [ue,un,uv]=uC(x,y,d,r,alpha)
C3 = 1-3*d.^2./r.^2;
ue = (1-alpha)*3*x.*d./r.^5;
un = (1-alpha)*3*y.*d./r.^5;
uv = (1-alpha)*C3./r.^3;

% Calculate partial derivatives with respect to X
function [rue_rx,run_rx,ruv_rx]=ruA_rx(x,y,d,r,alpha)
A3 = 1-3*x.^2./r.^2;
rue_rx = -(1-alpha)/2*A3./r.^3;
run_rx = (1-alpha)/2*3*x.*y./r.^5;
ruv_rx = (1-alpha)/2*3*x.*d./r.^5;

function [rue_rx,run_rx,ruv_rx]=ruB_rx(x,y,d,r,alpha)
A3 = 1-3*x.^2./r.^2;
rue_rx = (1-alpha)/alpha*A3./r.^3;
run_rx = -(1-alpha)/alpha*3.*x.*y./r.^5;
ruv_rx = -(1-alpha)/alpha*3.*x.*d./r.^5;

function [rue_rx,run_rx,ruv_rx]=ruC_rx(x,y,d,r,alpha)
A5 = 1-5*x.^2./r.^2;
C5 = 1-5*d.^2./r.^2;
rue_rx = (1-alpha)*3*d./r.^5.*A5;
run_rx = -(1-alpha)*15*x.*y.*d./r.^7;
ruv_rx = -(1-alpha)*3*x./r.^5.*C5;


% Calculate partial derivatives with respect to Y
function [rue_ry,run_ry,ruv_ry]=ruA_ry(x,y,d,r,alpha)
B3 = 1-3*y.^2./r.^2;
rue_ry = (1-alpha)/2*3*x.*y./r.^5;
run_ry = -(1-alpha)/2*B3./r.^3;
ruv_ry = (1-alpha)/2*3*y.*d./r.^5;

function [rue_ry,run_ry,ruv_ry]=ruB_ry(x,y,d,r,alpha)
B3 = 1-3*y.^2./r.^2;
rue_ry = -(1-alpha)/alpha*3.*x.*y./r.^5;
run_ry = (1-alpha)/alpha*B3./r.^3;
ruv_ry = -(1-alpha)/alpha*3*y.*d./r.^5;

function [rue_ry,run_ry,ruv_ry]=ruC_ry(x,y,d,r,alpha)
B5 = 1-5*y.^2./r.^2;
C5 = 1-5*d.^2./r.^2;
rue_ry = -(1-alpha)*15*x.*y.*d./r.^7;
run_ry = (1-alpha)*3*d./r.^5.*B5;
ruv_ry = -(1-alpha)*3*y./r.^5.*C5;


% Calculate partial derivatives with respect to Z
function [rue_rz,run_rz,ruv_rz]=ruA_rz(x,y,d,r,alpha)
C3 = 1-3*d.^2./r.^2;
rue_rz = -(1-alpha)/2*3*x.*d./r.^5;
run_rz = -(1-alpha)/2*3*y.*d./r.^5;
ruv_rz = (1-alpha)/2*C3./r.^3;

function [rue_rz,run_rz,ruv_rz]=ruB_rz(x,y,d,r,alpha)
C3 = 1-3*d.^2./r.^2;
rue_rz = (1-alpha)/alpha*3*x.*d./r.^5;
run_rz = (1-alpha)/alpha*3*y.*d./r.^5;
ruv_rz = -(1-alpha)/alpha*C3./r.^3;

function [rue_rz,run_rz,ruv_rz]=ruC_rz(x,y,d,r,alpha)
C5 = 1-5*d.^2./r.^2;
rue_rz = -(1-alpha)*3*x./r.^5.*C5;
run_rz = -(1-alpha)*3*y./r.^5.*C5;
ruv_rz = (1-alpha)*3*d./r.^5.*(2+C5);
